/* ************************************************************************** */
/** Descriptive File Name: stepper.c is the state control for a 4-wire
    stepper motor.

  @Company
    Digilent   
 
  @Author
    Dr. Richard W Wall
 
  @Date
    Created:	June 3, 2016
	Verified:	May 20, 2017
 
  @File Name
    stepper.c

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    4 wire and 5/6 wire stepper motor control.

  @Description
    This file provides the control of a stepper motor in full or half steps.
    This project specifically targets the Digilent motor available from
    http://store.digilentinc.com/stepper-motor/.

  ****   This application uses a peripheral bus clock set to 10MHz.  ****
    PBCLK set for 10 MHz, Set parameters in config_bits.h and hardware.h

 */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

// System included files
#include "hardware.h"
#include <plib.h>

// Application included files
#include "stepper.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* This file is for use with any 4W or 5/6 wire stepper motor provided 
 * the phasing is correct.
 * 
 * Digilent P/N 290-028 - http://store.digilentinc.com/stepper-motor/ 
 *  Phase    Wire color
 *  B1IN       Blue
 *  B2IN       Yellow
 *  A2IN       Orange
 *  A1IN       Pink
 *             Red - No connection
 */ 


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Public Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/** 
  @Function
    void steppep_init(void); 

  @Summary
    Initializes the PIC32 pins for output

  @Description
    The four stepper motor phase and mode pins are set as outputs and set to
    zero (off). 

  @Precondition
    None
    
  @Parameters
    @param param1 
        None
  @Returns
    None
  @Remarks
    All analog capable IO pins must have the analog functionality disabled
  @Example
    @code
    steppep_init(void); 
    @code
 */
void stepper_init(void) 
{
    MCInit();
}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/*  Stepper Motor Control  */

// *****************************************************************************

/** 
  @Function
    void stepper(int direction, int mode);
 
   @Summary
    Implements a single stepper motor step based on direction and mode.

  @Description
    Determines the stepper motor control code based upon the
    present state, direction, and mode. This rendition of the
    stepper motor FSM uses an algorithm to determine the next
    state that is representative of a ROM based FSM.
 
  @Precondition
    Processor pin must have been set for digital output. See stepper_init();
    
  @Parameters
    @param param1 
        Direction of rotation --> +1 or -1
    @param param2
        Step mode --> HALF STEP --> +1 or FULL STEP --> 2
 
  @Returns
    None
 
  @Remarks
    Stepper outputs must be bit banged and hence incorrect output codes may  
    be generated for some transitions. This function also has entry points to
    allow determining the number of steps per revolution by trial and error.
    This is nominally 1600 but can range from 1488 to 1712 and still be in the
    7% tolerance range.

  @Example
    @code
    stepper(CW, HALF_STEP); 
 */
void step(int direction, int mode)
{
#define REVS 65*25*10+5
    
static const unsigned char step_code[8] = {8, 9, 1, 5, 4, 6, 2, 10}; 
static unsigned int step_ptr = 0;   // Initialize pointer only on power up
static int revs = REVS;
    if(revs > 0)
    {
        revs--;
        step_ptr += (direction * mode);     // Compute step code pointer
        step_ptr &= sizeof(step_code)-1;	// force modulo 8

        AIN1(step_code[step_ptr] & 0x01);
        AIN2(step_code[step_ptr] & 0x02);
        BIN1(step_code[step_ptr] & 0x04);
        BIN2(step_code[step_ptr] & 0x08);
    }
    else
    {
        revs = REVS;
    }
}

/* *****************************************************************************
 End of File
 */
