/* ************************************************************************** */
/** Descriptive File Name Demonstration of LED blink routine based on a Timer
	interrupt.
    Lab2d.c
  @ Author
    Richard Wall
 
  @ Date
    Created:     September 21, 2016
    Verified:    May 16, 2017

  @Company
    Digilent Inc.

  @File Name
    lab2b.c

    @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library


  @Summary
    State based Timer based delays.

  @Description
    Delay routines that blink two LEDs based on the rate of Timer 1 interrupt.

  @Remarks
    Code based on https://www.ics.uci.edu/~givargis/pubs/C38.pdf 
 
    Uses C32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/*********************************************************************
 * The purpose of this example code is to demonstrate the use of 
 * peripheral library macros and functions supporting the PIC32MX general 
 * purpose Type A Timer 1.
 *
 * Platform:     Basys MX3A 
 *
 * Features demonstrated:
 *      - Timer configuration
 *      - Timer Interrupt flag polling
 *      - Timer Interrupt flag resetting
 *
 * Description:
 *         - This example polls the Timer 1 interrupt flag and toggles LED0 and LED1.
 *
 * Oscillator Configuration Bit Settings: config_bits.h
 * Basys MX3A hardware initialization is provided by Hardware_Setup.c 
 * Notes:
 *        - Peripheral Clock: PBCLK = 80E6 / 8 = 10E6 
 *        - Timer1 clock = T1CLK = PBCLK/T1_PRESCALE = 10E6/256 = 39062 Hz
 *        - To generate a 1 sec delay, PR1 = (T1CLK)-1 = (39062-1) = 39061
*
 ********************************************************************/
#include "config_bits.h"
#include "hardware.h"
#include <plib.h>

// Application constant
#define T1_TICK     GetPeripheralClock()/256 

// Application global variables
int BL_Clk = 0;

// Function prototype
// None - do not prototype ISRs

int main(void)
{
 enum BL_StateType {BL_On, BL_Off} BL_State;
    Hardware_Setup();       // Initialize Basys MX3 IO hardware
    setLED0(0);             //Initialize outputs

// Configure Timer1 to roll over once each second 
    OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_256, T1_TICK-1);
// set up the timer interrupt with a priority of 2 
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_2);

// Enable multi-vector interrupts 
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);  // Do only once
    INTEnableInterrupts();   // Do as needed for global interrupt control
    
    BL_State = BL_On;           // Initial state
    
    while (1) 
    {
        switch (BL_State)       // State actions
        {
            case BL_On:
                setLED0(1);     //Led_0 = 1;
                break;
            case BL_Off:
                setLED0(0);     //Led_0 = 0;
            break;
         } // State actions
        
        while (!BL_Clk);    // Wait for timeout
        {
            BL_Clk = 0;
        }
        
        switch (BL_State)  // Transitions
        {
            case BL_On:
                BL_State = BL_Off;
                break;
            case BL_Off:
                BL_State = BL_On;
                break;
        } // Transitions
    } // End of while(1)
    
    return (EXIT_FAILURE);  // System has failed if this line is reached
} // main 

/* START FUNCTION DESCRIPTION ********************************************
 * SYNTAX:         void __ISR(_TIMER_1_VECTOR, IPL2SOFT) Timer1Handler(void);
 * KEYWORDS:       Interrupt service routing, hardware delay
 * DESCRIPTION:    The LED1 is toggled and the global variable, BL_Clk is set
 *                 each time the interrupt occurs.
 * Parameters:     None
 * RETURN:         None
 * NOTES:          The Timer 1 interrupt flag must be reset.
 * END DESCRIPTION *******************************************************/
void __ISR(_TIMER_1_VECTOR, IPL2SOFT) Timer1Handler(void)
{
    invLED1();
    BL_Clk = 1;
    INTClearFlag( INT_T1 );                   // Clear the interrupt flag
}
/* End of lab2d.c  */
