/* ************************************************************************** */
/** Descriptive File Name - I2C communication implementation

  @ Author  
     Richard Wall
  
  @ Date: 
     Created:    August 29, 2014
     Revised:    June 21, 2016 - ported to Basys MX3
     Verified:	 May 17, 2017
	 
  @Company
    Digilent Inc

  @File Name
    I2C_lib.h

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    This file provides access to the I2C communications tasks. All functions
    require the specification of the I2C port either I2C1 or I2C2.

  @Description

**************************************************************************** */

#ifndef __I2CLIB_H__
    #define __I2CLIB_H__

    #define CCLK        (80000000)  // (8 * 10) MHz Osc on Basys MX3 
    #define PBCLK       (CCLK/8)    // Peripheral bus clock

//#define MAX_PAGES       64

    typedef struct 
    {
        I2C_MODULE i2c_channel; // I2C channel ? I2C1 or I2C2
        BYTE dev_id;            // I2C device ID
        BYTE reg_addr;          // Address of register to start
        BYTE block_size;        // Number of bytes to read or write
        BYTE *data;             // Byte pointer to data array
    } DATA_BLOCK;

#endif

/* Function Prototypes */
/* I2C_init FUNCTION DESCRIPTION ********************************************
 *
 * SYNTAX:     	void I2C_init(I2C_MODULE i2c_port, unsigned int bitrate);
 * KEYWORDS:	i2c, bit rate generator
 * DESCRIPTION:	Opens I2C channel 1 and sets the i2c bit rate based upon PBCLK
 * Parameter 1:	I2C_MODULE i2c_port: I2C1 or I2C2
 * Parameter 2:	I2C bit rate - must be less than 400000
 * RETURN VALUE: Operation success flag - TRUE if successful otherwise FALSE
 * NOTES:	The desired bit rate. Must be less than	400,000 or the bit rate
 *              of the slowest I2C device on the I2C bus.
END DESCRIPTION **********************************************************/
BOOL I2C_init( I2C_MODULE i2c_port, unsigned int bitrate);

/* I2C_ReadModbus FUNCTION DESCRIPTION ***************************************
 *
 * SYNTAX:      I2C_RESULT I2C_ReadModbus(MODBUS_DATA_BLOCK blk);
 * KEYWORDS:    Read Device, device address, memory address
 * DESCRIPTION:	Reads a I2C device using the Modbus protocol
 *
 * Parameter 1:	    typedef struct
 *                  {
 *                      I2C_MODULE i2c_channel;
 *                      BYTE dev_id;
 *                      BYTE reg_addr;
 *                      BYTE block_size;
 *                      BYTE *data;
 *                      } MODBUS_DATA_BLOCK;
 *
 * RETURN VALUE: I2C_RESULT - i2c transaction results
 * NOTES:	There is no restriction on the number of bytes that can
 * 		be read in a single operation provided that the block size is 
 *		less than the size of an unsigned integer data type.
 *
 *              The read operation receives the following data after sending
 *              the device ID with the READ bit set high:
 *                  READ code - 0x03
 *                  Number of bytes to read - N<10
 *                  [N data bytes]
 *                  2 CRC check bytes
 *              There is a NACK sent after the last CRC byte
END DESCRIPTION **************************************************************/
I2C_RESULT I2C_ReadBlk( DATA_BLOCK blk);

/* I2C_WrtiteBlk FUNCTION DESCRIPTION *************************************
 *
 * SYNTAX:      I2C_RESULT I2C_WrtiteBlk(DATA_BLOCK blk, BOOL RepStart);
 * KEYWORDS:	Write,  acknowledgement, write cycle complete
 * DESCRIPTION:	Writes a block of data to an I2C device
 *
 * Parameter 1:	    typedef struct
 *                  {
 *                      I2C_MODULE i2c_channel;
 *                      BYTE dev_id;
 *                      BYTE reg_addr;
 *                      BYTE block_size;
 *                      BYTE *data; // optional
 *                      } DATA_BLOCK;
 *
 * Parameter 2:     BOOL RepStart
 * RETURN VALUE: I2C_RESULT - i2c transaction results 
 * NOTES:		This write algorithm writes data in 64 byte boundries that
 *				is compatible with I2C EEPROM devices.
 * END DESCRIPTION **********************************************************/
I2C_RESULT I2C_WriteBlk( DATA_BLOCK blk);

/* I2C_TransmitOneByte *******************************************************
 *
 * Function:    BOOL I2C_TransmitOneByte( I2C_MODULE i2c_port, BYTE data )
 * Summary:     This transmits one byte to the EEPROM.
 * Description: This transmits one byte to the EEPROM, and reports errors for
 *              any bus collisions.
 * Precondition: The transfer must have been previously started.
 *
 * Parameters:  i2c_port    - I2C_MODULE
 *              data        - Data byte to transmit
 *
 * Returns:     TRUE    - Data was sent successfully
 *              FALSE   - A bus collision occurred
 *
 * Remarks:     This is a blocking routine that waits for the transmission to
 *              complete.
  ***************************************************************************/
I2C_RESULT I2C_ReceiveOneByte( I2C_MODULE i2c_port, BYTE *data, BOOL ack );

/* I2C_ReceiveOneByte *******************************************************
 *
 * Function:    I2C_RESULT I2C_ReceiveOneByte( I2C_MODULE i2c_port,
 *                                              BYTE *data, BOOL ack );
 * Summary:     This transmits one byte to the EEPROM.
 * Description: This transmits one byte to the EEPROM, and reports errors for
 *              any bus collisions.
 * Precondition: The transfer must have been previously started.
 *
 * Parameters:  i2c_port    - I2C_MODULE
 *              data        - Data byte to transmit
 *              ack         - TRUE - Send ACK
 *                          - FALSE - SEND NO ACK
 *
 * Returns:     TRUE    - Data was sent successfully
 *              FALSE   - A bus collision occurred
 *
 * Remarks:     This is a blocking routine that waits for the transmission to
 *              complete.
  ***************************************************************************/
BOOL I2C_TransmitOneByte( I2C_MODULE i2c_port, BYTE data );

/* End of I2C_lib.h */
