/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Digilent

  @Author
    Richard Wall

  @Date
    Created:    June 24, 2016
    Verified:   May 17, 2017

  @File Name
    MMA8652FC.c

  @Summary
    Library for MMA8652FC three axis accelerometer.

  @Description
 * This file contains functions to access and controls various aspects of the
 * MMA8652FC three axis accelerometer. Requires I2C primitive functions of 
 * Write and read block using a structure of the following format:
 *      typedef struct
 *      {
 *          I2C_MODULE i2c_channel;
 *          BYTE dev_id;
 *          BYTE reg_addr;
 *          BYTE block_size;
 *          BYTE *data;     // optionally used
 *      } DATA_BLOCK;
 * 
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "hardware.h"   // This file contains defititions that suppress warnings
#include <plib.h>       // Peripherial library
#include "MMA8652FC.h"  // Public file declarations
#include "I2C_lib.h"    // I2C Driver functions
#include "swDelay.h"

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
/** 
  @Function
    I2C_RESULT mma8562fc_init(void) 

  @Summary
    Initializes MMA8653FC three axis accelerometer.

  @Description
    Initialization sequence for MMA8653FC accelerometer using a series of
    I2C messages.

  @Parameters
    None

  @Returns
    I2C results - I2C_SUCCESS / I2C_FAILURE 

  @Remarks
    All I2C operations must be successful for this function to complete 
    correctly.  I2C communications must have been previously initialized.
***************************************************************************** */
I2C_RESULT mma8562fc_init(void)
{
BOOL okay;
BYTE xyz[8];
DATA_BLOCK xyz_blk;
I2C_RESULT i2c_ops;

// Constant device variables - not required to be changed after initialization
    xyz_blk.i2c_channel = I2C1;
    xyz_blk.dev_id = XYZ_ADDR;
    xyz_blk.data = xyz;
    msDelay(10);    
    
// Set to standby so device can be configured
    xyz[0] = 0x00;                      // Data to be written to register
    xyz_blk.block_size = 1;             // Number of bytes to write
    xyz_blk.reg_addr = XYZ_CTRL_REG1;   // Starting register address
    i2c_ops = I2C_WriteBlk(xyz_blk);    // Call block write
    msDelay(10);                        // Delay specified in data sheet

// Set to 2G Full scale mode
    if(i2c_ops == I2C_SUCCESS)
    {
        xyz[0] = 0x00;                      // Data to be written to register
        xyz_blk.block_size = 1;             // Number of bytes to write
        xyz_blk.reg_addr = XYZ_DATA_CFG;    // Starting register address
        i2c_ops = I2C_WriteBlk(xyz_blk);    // Call block write
        msDelay(1);                         // Delay specified in data sheet
    }
// Set F-Mode to 0
    if(i2c_ops == I2C_SUCCESS)
    {
        xyz[0] = 0x00;                      // Data to be written to register
        xyz_blk.block_size = 1;             // Number of bytes to write
        xyz_blk.reg_addr = XYZ_F_SETUP;     // Starting register address
        i2c_ops = I2C_WriteBlk(xyz_blk);    // Call block write
        msDelay(1);
    }

// Set to Active mode
    if(i2c_ops == I2C_SUCCESS)
    {
        xyz[0] = 0x01;                      // Data to be written to register
        xyz_blk.block_size = 1;             // Number of bytes to write
        xyz_blk.reg_addr = XYZ_CTRL_REG1;   // Starting register address
        i2c_ops = I2C_WriteBlk(xyz_blk);    // Call block write
        msDelay(10);                        // Delay specified in data sheet
    }
    
    return i2c_ops;
}

/** 
  @Function
    I2C_RESULT mma8562fc_xyz(int *status, int *x, int *y, int *z); 

  @Summary
    Reads the X, Y, and Z 12 bit parameters from the accelerometer

  @Description
    Reads the accelerometer data as an array of bytes and parses this array
    into the parameter values.

  @Parameters
    @param param1 - pointer to accelerometer status value
    @param param2 - pointer to accelerometer x axis value
    @param param3 - pointer to accelerometer y axis value
    @param param4 - pointer to accelerometer z axis value

  @Returns
    I2C results - I2C_SUCCESS / I2C_FAILURE 

  @Remarks
    Reports success or I2C operation as an indication of valid data.
***************************************************************************** */
I2C_RESULT mma8562fc_xyz(int *status, int *x, int *y, int *z)
{
BYTE xyz[8];
DATA_BLOCK xyz_blk;
I2C_RESULT i2c_ops;
int i;

	// set the I2C channel and addressing of the accelerometer
    xyz_blk.i2c_channel = I2C1;
    xyz_blk.block_size = 1;
    xyz_blk.dev_id = XYZ_ADDR;
    xyz_blk.data = xyz;
    xyz_blk.reg_addr = XYZ_STATUS;
    
    xyz_blk.block_size = 7;

	// read and process the I2C data
    i2c_ops = I2C_ReadBlk(xyz_blk);

    *status = (int) xyz[0];
    *x = (((int) (xyz[1]<<8)) + ((int) xyz[2])) >> 4;
    *y = (((int) (xyz[3]<<8)) + ((int) xyz[4])) >> 4;
    *z = (((int) (xyz[5]<<8)) + ((int) xyz[6])) >> 4;
    return i2c_ops;
}

/** 
  @Function
    I2C_RESULT mma8562fc_rd_reg(int reg, int *reg_value); 

  @Summary
    Reads a single byte register

  @Description
    Reads a single byte from the specified accelerometer register.

  @Parameters
    @param param1 - accelerometer register value
    @param param2 - pointer to accelerometer register value

  @Returns
    I2C results - I2C_SUCCESS / I2C_FAILURE 

  @Remarks
    Reports success or I2C operation as an indication of valid data.
***************************************************************************** */
I2C_RESULT mma8562fc_rd_reg(int reg, int *reg_value)
{
BYTE xyz[2];
DATA_BLOCK xyz_blk;
I2C_RESULT i2c_ops;

// Complete I2C interface structure
    xyz_blk.i2c_channel = I2C1;     // I2C CHannel
    xyz_blk.block_size = 1;         // Number of bytes in the block of data
    xyz_blk.dev_id = XYZ_ADDR;      // Accelerometer device address
    xyz_blk.data = xyz;             // Assign block pointer to memory array
    xyz_blk.reg_addr = reg;         // Register address
    
    i2c_ops = I2C_ReadBlk(xyz_blk); // Read single byte
    *reg_value = xyz[0];            // Transfer data to passed pointer
    
    return i2c_ops;
}

/* *****************************************************************************
 End of File
 */
