/* ************************************************************************** */
/** Descriptive File Name: Driver for SPI Flash Memory

   @Company
    Digilient
 
  @Author
    Richard Wall

  @Date
    Created:    August 11, 2013
    Verified:   May 20, 2017

  @File Name
    spi.c

  @Summary
    Serial peripheral interface (SPI) communications.

  @Description
    SPI Initialization and communication specifically targeting the Basys MX3
    using SPI channel 1.
 ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

// System included files
#include "hardware.h"    // Basys MX3 pin definitions 
#include <plib.h>

// Platform common included files
#include "spi.h"

// *****************************************************************************
/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/** 
  @Function
    unsigned int init_SPI1(void)  

  @Parameters
    None
    
  @Returns
    Actual SPI bit rate

  @Summary
    Initialization of SPI communications for read and writes.

  @Remarks
    PIC32MX3xx PPS pin RF7 is mapped to SDI1 and RF2 is mapped to SDO1. SDI1
    chip enable is not assigned here.
 */
unsigned int init_SPI1(void)
{
unsigned int pbFreq;
unsigned int SPI_Clk_Freq;
unsigned int berg_val;

// Map PPS pins to SDI1
    SDI1R = 0x0F;       // Map SDI1 to RF7 - Input (MISO))
    RPF2R = 0x08;       // MAP SDO1 to RF2 - Output (MOSI))
//  SCK1 has fixed assignment to RF6

// Initialize the direction of the SPI interface signals.  The device SS
// is not assigned in this initialization since the pin assignment can be
// hardware dependent. */
    SDI1_TRIS = 1;      // FLASH SO 
    SDO1_TRIS = 0;      // FLASH SI
    SCK1_TRIS = 0;      // FLASH SCK
    SPI1_SDO = 0;
    SPI1_SCK = 0;
    
// Peripheral Bus Frequency = System Clock / PB Divider 
// PB Frequency can be maximum 40 MHz *
    pbFreq = GetPeripheralClock();
// Compute proper BERG value for specified SPI bit rate
    berg_val = SpiBrgVal(pbFreq, MAX_SPI_CLK_FREQ);
// Compute actual SPI bit rate
    SPI_Clk_Freq = pbFreq / ( 2 * (berg_val+1) );
    
// Enable SPI1, Set to Master Mode & Set CKE bit 1 for SPI MODE 0
// Serial output data changes on transition from active clock state to Idle 
// clock state. Idle clock state is low.
//  SpiChnOpen(1, config, berg_val );  This PLIB function does not set bit rate.
    SPI1BRG = berg_val;         // Set SPI bit rate
    SPI1CONbits.MSTEN = 1;      // SPI Master enable
    SPI1CONbits.CKE = 1;        // Set for SPI Mode 0
    SPI1CONbits.ON = 1;         // Enable SPI1
    return SPI_Clk_Freq;
}


/** 
  @Function
    BYTE spiXfer(BYTE data_out)  

  @Parameters
    @Parameter 1: data to be sent out the SPI shift register.
    
  @Returns 
    Data received into the shift register

  @Summary
    Completes a BYTE send and receive.

  @Remarks
    PIC32MX3xx PPS pin RF7 is mapped to SDI1 and RF2 is mapped to SDO1. SDI1
    chip enable is not assigned here.
 */
BYTE spiXfer(BYTE data_out)
{
    SpiChnPutC(1, data_out);
    usDelay(1);
    return SpiChnGetC(1);
}

/** 
  @Function
    void close_SPI1(void)  

  @Parameters
    None
    
  @Returns 
    None

  @Summary
    Disables SPI1.

  @Remarks
    None
 */
void close_SPI1(void)
{
    SPI1CONbits.ON = 0;         // Disable SPI1    
}

/* End of SPI.c */
