/* ************************************************************************** */
/** Descriptive File Name: Driver for Flash Memory

   @Company
    Digilient
 
  @Author
    Richard Wall

  @Date
    July 8, 2016
    Rev. August 8, 2016
    Rev. October 31, 2016

  @File Name
    flash_mem.c

  @Summary
    Software driver for reading and writing to FLASH memory using SPI interface.

  @Description
    Hardware handler for the S25FL132 flash memory device.
    See data sheet posted http://www.cypress.com/file/196886/download

     
  @Remarks
      This FLASH interface uses the include SPI hardware functions.
      See data sheet posted http://www.cypress.com/file/196886/download*/
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

// System included files
#include "hardware.h"
#include <plib.h>
#include <stdio.h>

// Platform common included files
#include "hardware.h"
#include "swDelay.h"

// Application included files
#include "flash_mem.h"
#include "SPI.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/*  Hardware driver for Flash Memory  */

unsigned char spi_rd[10], spi_wr[10];

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

static void SPIFLASH_SendByte(unsigned char bVal);
static void SPIFLASH_SendOneByteCmd(unsigned char bCmd);
static BYTE SPIFLASHReadStatus(void);

/*  SPIFLASH support functions.  */

/* ************************************************************************** */
/** 
  @Function
    static BYTE SPIFLASHReadStatus(void) 

  @Summary
    Reads the SPI FLASH volatile status bits.

  @Description
    Reads the FLASH SR1 register and masks for the SPIFLASH_BUSY_STATUS_BIT
    and SPIFLASH_WEL_STATUS_BIT.

  @Precondition
    SPI channel must have been initialized.

  @Parameters
    None
    
  @Returns
    BYTE with SPIFLASH_BUSY_STATUS_BIT and SPIFLASH_WEL_STATUS_BIT in the two
    LSB positions.
  
  @Remarks
    This function actually reads the SR1 register twice and used the second
    reading
  @Example
    @code
    {
        BYTE status;
        status = SPIFLASHReadStatus();
    }
 */
static BYTE SPIFLASHReadStatus(void)
{
BYTE wr[5] = {0}, rd[5] = {0};
    
    wr[0] = SPIFLASH_CMD_RDSR_1;    
    SPIFLASH_TrasferBytes(3, rd, wr);
    return (rd[2] & (SPIFLASH_BUSY_STATUS_BIT | SPIFLASH_WEL_STATUS_BIT));
}

/** 
  @Function
    static void SPIFLASH_SendByte(unsigned char bVal) 

  @Summary
    Sends one SPI byte.

  @Description
    Sends one byte of data but does not receive the Rx byte.

  @Precondition
    SPI channel must have been initialized.
    Chip must have been enable before calling this function.
    Chip Enable signal must be asserted high outside this function

  @Parameters
    @param param1 BYTE variable to be transmitted.
    
  @Returns
    None
  
  @Remarks
    Function blocks until buffer is empty. BYTE data type is the same as 
    unsigned char data type.

  @Example
    @code
    {
        SPIFLASH_SendByte(unsigned char bVal);
    }
 */
static void SPIFLASH_SendByte(unsigned char bVal)
{
    while(!SPI1STATbits.SPITBE);	// wait for TX buffer to be empty
    SPI1BUF = bVal;
    while(!SPI1STATbits.SPIRBF);	// wait for TX buffer to be empty
}

/** 
  @Function
    static void SPIFLASH_SendOneByteCmd(unsigned char bCmd) 

  @Summary
    Sends one SPI byte command.

  @Description
    Sends one byte of data but does not receive the Rx byte. Asserts CS low
    before transmission and high afterwards.

  @Precondition
    SPI channel must have been initialized

  @Parameters
    @param param1 BYTE variable to be transmitted.
    
  @Returns
    None
  
  @Remarks
    BYTE data type is the same as unsigned char data type

  @Example
    @code
        unsigned char bVal;
        SPIFLASH_SendOneByteCmd(bVal);
 */
static void SPIFLASH_SendOneByteCmd(unsigned char bCmd)
{
    SPIFLASH_SEL = 0;              // Select device
    SPIFLASH_SendByte(bCmd);
    SPIFLASH_SEL = 1;              // Remove device select
}
    
// *****************************************************************************
/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/** 
  @Function
    unsigned int flash_mem_init(void)  

  @Parameters
    None
    
  @Returns
    None

  @Summary
    Initialization of SPI communications and sets up flash memory for read
    and writes.

  @Remarks
    The SPI pins are configured in the "init_spi()" but the Flash CS pin is 
    configured here.
 */
unsigned int flash_mem_init(void) 
{
unsigned int bitrate;

    bitrate = init_SPI1();
    SPIFLASH_CS_TRIS = 0;       // Set FLASH select as output
    SPIFLASH_SEL = 1;           // Reset FLASH chip select 
    return bitrate;             // Has no meaning at this time
}

/** 
  @Function
    void SPIFLASH_ReleasePowerDownGetDeviceID(BYTE *rd);  
  
  @Parameters
    @param param1 BYTE pointer for to buffer.
    
  @Returns
    None

  @Summary
    Releases Power Down and reads device ID

  @Remarks
    Uses globally declared write buffer and referenced read buffer to return
    status bytes.
    <p>
    This is a specific command function tailored for this command only. Generic
    FLASH command and status operations are provided by the "SPIFLASH_CmdSet"
    function.
    <p>
    This function was for initial testing only. The need for it has been 
    replaced by void SPIFLASH_CmdSet(BYTE cmd, BYTE *rd, int len); See below.
 */
void SPIFLASH_ReleasePowerDownGetDeviceID(BYTE *rd)
{  
    spi_wr[0] = SPIFLASH_CMD_PWRDWN_DEVID;
    spi_wr[1] = 0;
    spi_wr[2] = 0;
    spi_wr[3] = 0;
    spi_wr[4] = 0;
    SPIFLASH_TrasferBytes(5, rd, spi_wr);
}

/** 
  @Function
    void SPIFLASH_CmdSet(BYTE cmd, BYTE *rd, int len)  
  
  @Parameters:
    @param param1 BYTE value of the command.
    @param param2 BYTE pointer for to buffer.
    @param param3 integer value of length of buffer read.
    
  @Returns
    None

  @Summary
    Sends FLASH command and reads FLASH data

  @Remarks
    Uses globally declared write buffer and referenced read buffer to return
    status bytes.
 */
void SPIFLASH_CmdSet(BYTE cmd, BYTE *rd, int len)
{  
int i;
    if(len <= sizeof(spi_wr)-1)
    {
        spi_wr[0] = cmd;
        for(i=1; i<len; i++)
        {
            spi_wr[i] = 0;
        }
        SPIFLASH_TrasferBytes(len, rd, spi_wr);
    }
}


/** 
  @Function
    void SPIFLASH_TrasferBytes(int nBytes, unsigned char *pbRdData, 
                           unsigned char *pbWrData)
  @Summary
    Writes and Reads a specified number of bytes. The write hearer (command and 
    starting address) must be contained at the beginning the pbWrData array.
    This transfer is limited to addresses with in a page and maximum number of
    of 256 bytes.  
 
  @Remarks
    The BYTE write waits until the SPI transmission is complete before reading 
    the SPI Buffer.
 */
void SPIFLASH_TrasferBytes(int nBytes, unsigned char *pbRdData, 
                           unsigned char *pbWrData)
{
    int i;
    SPIFLASH_SEL = 0; // Activate CS
    for(i = 0; i< nBytes; i++)
    {
        SPIFLASH_SendByte(pbWrData[i]); // Write byte to SPI FLash 
        pbRdData[i] = SPI1BUF;
    }
    SPIFLASH_SEL = 1; // Deactivate CS
}

/** 
  @Function
    void SPIFLASH_ReadData(int nBytes, unsigned char *pbRdData, 
                           unsigned int data_addr)
  @Summary
    Reads a specified number of bytes. The address pointer must first
    be written to the FLASH.  
  @Remarks
    There is no limit on number of bytes to be read.
 */
void SPIFLASH_ReadData(int nBytes, unsigned char *pbRdData, 
                           unsigned int data_addr)
{
BYTE wr_hdr[5];                             // Header buffer
BYTE dump_data;                             // Non-consequential returned data
int i;                                      // General counter

    wr_hdr[0] = SPIFLASH_CMD_READ;          // Setup Read cmd and address
    wr_hdr[1] = (BYTE) (data_addr >> 16);
    wr_hdr[2] = (BYTE) (data_addr >> 8);
    wr_hdr[3] = (BYTE) (data_addr);
    wr_hdr[4] = 0;         // Byte written while reading - ignored by slave
    
    SPIFLASH_SEL = 0;                       // Activate CS
    for(i=0; i<4; i++)                      // Send Read cmd and address
    {
        SPIFLASH_SendByte(wr_hdr[i]);
        dump_data = SPI1BUF;                // Received value is discarded
    }
    for(i=0; i<nBytes; i++)                 // Read data bytes
    {
        SPIFLASH_SendByte(wr_hdr[4]);       // Byte value sent does not matter
        *pbRdData++ = SPI1BUF;              // Read data shifted in during write
    }
    SPIFLASH_SEL = 1;                       // deactivate CS    
}

/** 
  @Function
    int SPIFLASH_WritePage(int nBytes, unsigned char *pbWrData, 
                           unsigned char data_addr)
  @Summary
    Writes a specified number of bytes to an erased page or sub page of FLASH
    memory. This is a blocking function.
   
  @Remarks
    Memory can only be programmed that has been previously erased (0xFF). A 
    maximum of 256 bytes can be written and can never across a page boundary.
    (Meaning advancing LSB address byte from 0xFF to 0x00.) This function
    performs the following operations:

        1. Checks to for number of bytes to be written less than 256 and will
           not attempt to write across a page boundary (xxFF to xx00).
        2. Reads the FLASH page or portion of FLASH page into temporary buffer
        3. Checks to see that portion of FLASH memory has been erased
        4. Returns an error of 1 if not erased otherwise continues
        5. Sets WEL bit with SPIFLASH_CMD_WREN command.
        6. Polls the SR1 status register until the WEL bits is read as 1 and
           the BUSY bit is read as zero.
        7. Sets the write header buffer for SPIFLASH_CMD_WRITE command followed
           by the 24 bit starting address.
        8. Sets CS low.
        8. Writes the four bytes of the header buffer followed by 1 to 256 
           bytes of data subject to the constraints described above.
        10. Sets CS high
        11. Clears WEL bit with SPIFLASH_CMD_WRDI command.

 */
int SPIFLASH_WritePage(int nBytes, unsigned char *pbWrData, 
                           unsigned int data_addr)
{
BYTE wr_hdr[5] = {0};	// Used for command and address
BYTE rd_hdr[5] = {0};
BYTE tmpBuffer1[256] = {0};	// Read buffer for checking page erased
unsigned int pageAddr;	// Start write address
int error = 0;          // Process error
BYTE chk_flag = 0xFF;	// Check erased flag
int i;                  // General index
BYTE FLASHStatus;		// Flash memory busy flag


// Test page constraint
    if(((data_addr & 0x000000FF) + nBytes) > 256)
    {
        error = 1;  // Write beyond page boundary
    }      
    else
    {

// Read existing page contents
        SPIFLASH_ReadData(nBytes, tmpBuffer1, data_addr);

// Check for memory erased
        for(i=0; i<nBytes; i++)    
        {
            chk_flag &= tmpBuffer1[i];
        }
        if(chk_flag != 0xFF)
        {
            error = 1;
            return error;
        }
              
        wr_hdr[0] = SPIFLASH_CMD_WREN;  // Unlock FLASH for writing
        SPIFLASH_TrasferBytes(1, rd_hdr, wr_hdr);
      
// Poll Status Register 1 until WEL bit is set and BUSY flag is reset.
        do {
            FLASHStatus = SPIFLASHReadStatus();
        } while(FLASHStatus != SPIFLASH_WEL_STATUS_BIT);

// Setup write header
        wr_hdr[0] = SPIFLASH_CMD_WRITE;
        wr_hdr[1] = (BYTE) (data_addr >> 16);
        wr_hdr[2] = (BYTE) (data_addr >> 8);
        wr_hdr[3] = (BYTE) (data_addr);
        wr_hdr[4] = 0;
    
        SPIFLASH_SEL = 0;                   // Activate CS
        for(i=0; i<4; i++)                  // SPI Write command and address
        {
            SPIFLASH_SendByte(wr_hdr[i]);
            rd_hdr[0] = SPI1BUF;            // Ignore returned bytes
        }
        for(i=0; i<nBytes; i++)             // Write data
        {
            SPIFLASH_SendByte(*pbWrData++);
            rd_hdr[0] = SPI1BUF;            // Ignore returned bytes
        }
        SPIFLASH_SEL = 1;                   // deactivate CS

        wr_hdr[0] = SPIFLASH_CMD_WRDI;      // Disable SPI FLash write
        
        SPIFLASH_TrasferBytes(1, rd_hdr, wr_hdr);
        do {
            FLASHStatus = SPIFLASHReadStatus();
        } while(FLASHStatus != 0);  // Wait until write is complete

    }
    return error;
}

/** 
  @Function
    int SPIFLASH_WriteData(int len, BYTE *data, unsigned int addr)  

  @Summary
    Writes an arbitrary number of data bytes to an arbitrary starting address.

  @Remarks
    If the number of data bytes exceeds 256 bytes or the number of bytes will
    result in attempting to write across a page boundary, multiple page writes
    are necessary.
 */
int SPIFLASH_WriteData(int nBytes, BYTE *pbWrData, unsigned int flashAddr)
{
int error = 0;
unsigned int pageAddrS;     // Page start programming address
unsigned int pageAddrE;     // Page end programming address
unsigned int pageAddrL;     // Last Page end programming address
unsigned int pageBytes;     // Bytes to program in current page
unsigned int nPages;
BYTE *dataPtr;              // Updated data array pointer

    dataPtr = pbWrData;  
//    nPages = (nBytes % SPIFLASH_PAGE_SIZE) + 1;
    pageAddrS = flashAddr;
    pageAddrL = pageAddrS + nBytes;
    
    while((pageAddrS < pageAddrL) && !error)
    {
        pageAddrE = (pageAddrS & 0xFFFFFF00) + SPIFLASH_PAGE_SIZE;
        if(pageAddrE >= pageAddrL)
        {
            pageAddrE = pageAddrL; 
        }
        pageBytes = pageAddrE - pageAddrS;
        error |= SPIFLASH_WritePage(pageBytes, dataPtr, pageAddrS);
        pageAddrS = pageAddrE;
        dataPtr += pageBytes;
    }
    
    return error;
}

/** 
  @Function
    int SPIFLASH_Erase(BYTE cmd, unsigned char addr)
 * 
  @Summary
    Erases sector of FLASH memory. This is a blocking function.
   
  @Remarks
    Possible values for cmd:
        SPIFLASH_CMD_ERASE_4K       // SPI Flash opcode: 4KByte sector erase
        SPIFLASH_CMD_ERASE_32K      // SPI Flash opcode: 32KByte block erase
        SPIFLASH_CMD_ERASE_SECTOR   // SPI Flash opcode: 64KByte block erase
        SPIFLASH_CMD_ERASE_ALL      // SPI Flash opcode: Entire chip erase
    See additional notes on SPIFLASH_WritePage.
 */
void SPIFLASHErase(BYTE cmd, unsigned int Addr)
{
BYTE wr_hdr[4] = {0};
BYTE rd_hdr[4] = {0};
BYTE FLASHStatus;
int i;
int sBytes = 1;             // Default number of header bytes 

    wr_hdr[0] = SPIFLASH_CMD_WREN;  // Unlock FLASH for writing
    SPIFLASH_TrasferBytes(1, rd_hdr, wr_hdr);

    do {                            // Wait for SPI FLAS ready for writing
        FLASHStatus = SPIFLASHReadStatus();
    } while(FLASHStatus != SPIFLASH_WEL_STATUS_BIT);
    
    wr_hdr[0] = cmd;                    // Setup write header
    if(cmd != SPIFLASH_CMD_ERASE_ALL)   // Address not required for chip erase
    {
        wr_hdr[1] = (BYTE) (Addr >> 16);
        wr_hdr[2] = (BYTE) (Addr >> 8);
        wr_hdr[3] = (BYTE) (Addr);
        sBytes = 4;
    }

    SPIFLASH_SEL = 0;                   // Activate CS
    for(i=0; i<sBytes; i++)             // SPI Write command and address
    {
        SPIFLASH_SendByte(wr_hdr[i]);
        rd_hdr[0] = SPI1BUF;            // Ignore returned bytes
    }
    SPIFLASH_SEL = 1;                   // deactivate CS  
    
    do {                                // Wait until erasure complete
        FLASHStatus = SPIFLASHReadStatus();
    } while(FLASHStatus != 0);

//    wr_hdr[0] = SPIFLASH_CMD_WRDI;  // Lock FLASH for writing
//    SPIFLASH_TrasferBytes(1, rd_hdr, wr_hdr);    
}

/* *****************************************************************************
 End of File
 */
