/* ************************************************************************** */
/** Descriptive File Name - Stepper motor control
 
  @Author
    Richard W Wall

  @Company
    Digilent Inc.  
 
  @Date
    Created:    June 2, 2016
    Verified:   May 20, 2017

  @File Name
    stepper.h

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    4 wire and 5/6 wire stepper motor control.

  @Description
    This file provides the definitions and interface control of a stepper motor 
    in full or half steps.
 */
/* ************************************************************************** */

#ifndef _STEPPER_MOTOR_H_    /* Guard against multiple inclusion */
#define _STEPPER_MOTOR_H_


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */

/* TODO:  Include other files here if needed. */

#include "hardware.h"
#include <plib.h>
    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    /*  OUtput pin assignments.
     */
   
    #define A1IN1bit        BIT_3       //RB3
    #define A2IN1bit        BIT_8       //RE8
    #define B1IN1bit        BIT_9       //RE9
    #define B2IN1bit        BIT_5       //RB5

    #define STEPPER_MODE_DIR 	PORTSetDigitalOut(IOPORT_F, BIT_1) //RF1  
    #define STEPPER_MODE(a)    	PORTSetBits(IOPORT_F, BIT_1)       //RF1  

    #define SM_FULL_STEP    (int) 2
    #define SM_HALF_STEP    (int) 1
    #define SM_CW           (int) 1
    #define SM_CCW          (int) -1

    #define STEPSPREREV         48
    #define RPM2MSPERHALFSTEP   6250
    #define T2_TICK  GetPeripheralClock()/10000  // Timer 2 ticks per / ms

#endif // _STEPPER_MOTOR_H_

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /*    4 wire and 5/6 wire stepper motor control. */

    // *****************************************************************************
    /**
    /** 
  @Function
    void steppep_init(void); 

  @Summary
    Initializes the PIC32 pins for output

  @Description
    The four stepper motor phase and mode pins are set as outputs and set to
    zero (off). 

  @Precondition
    None
    
  @Parameters
    @param param1 
        None
  @Returns
    None
  @Remarks
    All analog capable IO pins must have the analog functionality disabled
  @Example
    @code
    steppep_init(void); 
 */
void stepper_init(void);

/** 
  @Function
    void stepper(int direction, int mode);
 
   @Summary
    Implements a single stepper motor step based on direction and mode.

  @Description
    Determines the stepper motor control code based upon the
    present state, direction, and mode. This rendition of the
    stepper motor FSM uses an algorithm to determine the next
    state that is representative of a ROM based FSM.
 
  @Precondition
    Processor pin must have been set for digital output. See stepper_init();
    
  @Parameters
    @param param1 
        Direction of rotation --> +1 or -1
    @param param2
        Step mode --> HALF STEP --> +1 or FULL STEP --> 2
 
  @Returns
    None
 
  @Remarks
    Stepper outputs must be bit banged and hence incorrect output codes may  
    be generated for some transitions.

  @Example
    @code
    stepper(CW, HALF_STEP); 
 */
void step(int dir, int mode);

/* *****************************************************************************
 End of File
 */
