/* ************************************************************************** */
/** Descriptive File Name - Control of the IrDA module

  @Company
    Digilent

  @ Author
    Richard Wall
 
  @Date
    Created:    August 16, 2016
    Verified:   May 17, 2017
  
  @File Name
    irda.c

  @Summary
    IrDA driver functions.

  @Description
    Decodes NEC or SIRC IrDA protocols. See Documentation.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

// System header files
#include "hardware.h"
#include <stdio.h>      /* Required for printf */
#include <plib.h>

// Application header files
#include "irda.h"
#include "swDelay.h"
//#include "lab5.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

unsigned int new_key;   // New key decoded flag
unsigned int key;       // Key value

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/**
  @Function
    void idra_nec_send_sync(void); 

  @Summary
    Initializes the IrDA Transceiver 

  @Description
    Turns on 38KHz pulse for 9 ms and off for 4.5 ms
    <p>
    PORT B pin 7 is used for the IrDA Tx line and is pulse width modulated
    using OC5.
    <p>
    This version is written for the NEC IrDA protocol
  @Precondition
    The function irda_init must be executed to map the PPS pin for IR_TX.
     
  @Parameters
    None
    
  @Returns
    None

  @Remarks
    None

  @Example
    @code
      {
        irda_init();
      }
 *************************************************************************** */
static void idra_nec_send_sync(void);

/******************************************************************************/
/**
  @Function:
    void idra_nec_send_bit(int sBit)

  @Summary:
    Sends a 38KHz pulse train for a 1 or 0

  @Description:
    Generate a 1 or 0 NEC encoded IrDA pulse train using PWM output . See 
    http://www.mcselec.com/index.php?option=com_content&task=view&id=223
    and http://www.vishay.com/docs/80071/dataform.pdf.

  @Precondition:
    IrDA initialization that maps PPS pin RPB7 to OC5
 
  @Paramteters
    @Parameter 1: bit value

  @Returns:
    None

  @Remarks:
    None

  @Example:   
    @code
     {
        idra_nec_send_bit(sBit);
     }

  *****************************************************************************/
static void idra_nec_send_bit(int sBit);

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/**
  @Function
    void irda_init(void); 

  @Summary
    Initializes Port B pin 6 as the IrDA Rx 

  @PARAMETERS:
    None
 
  @RETURN VALUE:
    None

  @Remarks
    Although the PIC32MX3xx data sheet discusses an UART mode that supports IrDA 
    it appears that the protocol Microchip presents is not one of the commercial
    control IrDA formats. See 
    http://www.mcselec.com/index.php?option=com_content&task=view&id=223 and
    http://www.vishay.com/docs/80071/dataform.pdf.

 * END DESCRIPTION **********************************************************/
void irda_init(void)
{
int i;
// IrDA Power Down Control
    TRISGCLR = BIT_1;       // Set IR_pdown as output();
    LATGCLR =  BIT_1;       // Set IR_pdown(0);

//  tris_IR_TX();           // Set direction for IrDA Rx
//  tris_IR_RX();          // Set direction for IrDA Tx
    
// Set up IrDA TX interface
    PORTSetPinsDigitalIn(IOPORT_B, BIT_7); // IR_TX - Tristate LED output
    RPB7R = 0b00001011;     // Mapping OC5 to RPB7
   
    OpenTimer2((T2_ON | T2_SOURCE_INT | T2_PS_1_1), IRDA_38KHZ_GEN);
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_3);
    
    OpenOC5((OC_ON|OC_TIMER_MODE16|OC_TIMER2_SRC|OC_PWM_FAULT_PIN_DISABLE), 
            IRDA_38K_IDLE, IRDA_38K_IDLE);
   
// Set up IrDA RX interface
    INT1R = 0b00000101;     // Mapping IrDA Rx to RPB6 --> INT1
    PORTSetPinsDigitalIn(IOPORT_B, BIT_6); // IR_RX
// Set up INT1 for negative edge triggering
    IEC0bits.INT1IE = 0;    // Disable INT1
    IPC1bits.INT1IP = 2;    // Set Interrupt 1 for priority level 2
    IPC1bits.INT1IS = 0;    // Set Interrupt 1 for sub-priority level 0
    INTCONbits.INT1EP = 0;  // Set for falling edge
    IFS0bits.INT1IF = 0;    // Clear the INT1 interrupt flag
    IEC0bits.INT1IE = 1;    // Enable INT1

    printf("\n\rIrDA Receiver ready\n\n\r");

    // Enable global nested interrupts    
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR); // Execute only once
    INTEnableInterrupts();  // Execute as needed
}

/**************************************************************************** */
/**
  @Function:
    int irda_nec(unsigned int dt_1)

  @Summary:
    Decodes the IrDA NEC protocol

  @Description:
    This ISR decode 
    the NEC IrDA protocol for control devices. See 
    http://www.mcselec.com/index.php?option=com_content&task=view&id=223 and
    http://www.vishay.com/docs/80071/dataform.pdf.

  @Precondition:
        IrDA initialization that maps PPS pin RPB6 to INT1
  @Parameters
    @Parameter 1: Interval period

  @Returns:
    Number of bits decoded.

  @Remarks:
        Modifies global variable "key" and "new_key"

  @Example
    @code
      {
        bit_cnt = irda_nec( dt );
      }
    
  *****************************************************************************/
int irda_nec(unsigned int dt_1)
{
static unsigned int code = 0;
static int code_idx = 0;
static unsigned int dt_sync, dt_zero, dt_one;   // Used for test only

    if(dt_1 > 175000 && dt_1 < 185000)      // 180499    sync gap
    {
        dt_sync = dt_1;                     // Used for test only
// ========================  Test outputs ===============================
        BIN2(0);                            // Reset BIN2 output
        BIN1(1);                            // Set BIN1 output
        dt_zero=0;
        dt_one=0;
// ======================================================================
        code = 0;
        code_idx = 0;

    }
//    if(dt_1 > 22700 && dt_1 < 22900)    // 22852    zero gap
    if(dt_1 > 22000 && dt_1 < 25000)    // 22852    zero gap
    {
// ========================  Test outputs ===============================
        if(dt_1 > dt_one)
        {
            dt_one = dt_1;                  // Used for test only
        }
        BIN2(1);                            // Set BIN2 output
        BIN1(0);                            // Reset BIN1 output
// ======================================================================
        code = code << 1;
        code_idx++;
    }        
    if(dt_1 > 66000 && dt_1 < 70000)    //67780     one gap
    {
// ========================  Test outputs ===============================
        if(dt_1 > dt_zero)
        { 
            dt_zero = dt_1;                 // Used for test only
        }
        BIN2(0);                            // Reset BIN2 output
        BIN1(0);                            // Reset BIN1 output
// ======================================================================
        code = code << 1;
        code |= 1;
        code_idx++;
    }        
    if(code_idx >= 32)
    {
        new_key = 1;                    // Set new key flag
        key = code;                     // Set key value
        code_idx = 0;                   // Reset bit counter
    }
    return code_idx;
}

/*******************************************************************************
  @Function:
    unsigned int irda_sirc(unsigned long dt_1)

  @Summary:
    Decodes IrDA Pulse-Length Coding. ALso known as SIRC infrared control code

  @Description:
    IrDA Pulse-Length Coding is described at following web cites.
        1. http://www.vishay.com/docs/80071/dataform.pdf
        2. http://www.st.com/content/ccc/resource/technical/document/application\
           note/c7/d1/63/f7/80/06/41/a4/CD00267896.pdf/files/CD00267896.pdf/jcr:\
           content/translations/en.CD00267896.pdf
 
  @Precondition:
    IrDA initialization that maps PPS pin RPB6 to INT1
 
  @Parameter 1: Interval period

  @Returns:
    Number of bits decoded.

  @Example:
    @code
        bit_cnt = unsigned int sirc( dt );

  @Remarks:
    The IrDA Pulse Length coding repeats the 20 bit code for the duration of the 
    key held depressed 

    The last bit is not decoded because the bit value is in the length of the
    pulsing stream.  The detection uses the pulsing stream plus the inter-bit
    gap interval.  Since the next interrupt occurs at the start of the new
    key press, the last bit exceeds the ONE or ZERO bit timing. By observation,
    the last bit is always set to ONE. Hence when 19 bits are received, a ONE
    bit is added to the code.
  *****************************************************************************/
unsigned int irda_sirc(unsigned long dt_1)
{
static unsigned int code = 0;       // Decoded bits
static unsigned long dt_acc;        // Delta times between interrupts
static unsigned int bitCount = 0;   //Number of decoded bits
static int pulse = 0;               // Stream state machine

    if(dt_1 > 117300)
    {
        pulse = 0;                  // Reset SM is not a valid pulse interval
    }

    switch(pulse)
    {
        case 0:
            code = 0;                   // Initialize code word
            dt_acc = 0;                 // Initialize interval accumulator
            pulse = 1;                  // Advance to state 1
        case 1:
            if(dt_1 > 900 && dt_1 < 1100)   // 1001     pulse period
            {
                dt_acc += dt_1;         // Add interval to signal length
                pulse = 2;              // Advance to leader search state
            }
            else
            {
                pulse = 0;              // Invalid carrier frequency
            }
            break;
        case 2:
            dt_acc += dt_1;             // Add interval to signal length
  //117198 - end of sync signal - and valid gap interval
            if(dt_acc > 117100 && dt_acc < 117300 && (dt_1 > 20000))
            {
                dt_acc = 0;             // Reset interval accumulator
                bitCount = 0;           // Reset bit counter
                pulse = 3;              // Start of data stream
            }
            if(dt_acc > 118000)
            {
                pulse = 0;              // Reset SM is not a valid pulse
            }
            break;
        case 3:
            dt_acc += dt_1;             // Add interval to signal interval
            if(dt_1 > 2000)             // Validate gap interval
            {
// One code - 70068 = 1.7594ms
                if(dt_acc > 70000 && dt_acc < 72000)
                {
                    code = code << 1;   // Shift code word left
                    code |= 1;          // Add one to LSB of code
                    dt_acc = 0;         // Reset interval accumulator
                    bitCount++;         // Increment bit counter
                } 
// Zero code - 45043 = 1.1449ms            
                if(dt_acc > 45000 && dt_acc < 47000) 
                {
                    code = code << 1;       // Shift code word left
                    dt_acc = 0;             // Reset interval accumulator
                    bitCount++;             // Increment bit counter
                }                
            }
            if(bitCount > 18)
            {
                code = code << 1;       // Add the final undetectable bit. 
                code |= 1;
                key = code;             // Transfer key code to global buffer
                new_key = 1;            // Set new key flag to signal a new key 
                pulse = 0;              // Reset SM
            }
            break;
    }
    return bitCount;
}

/*******************************************************************************
    @Function:
        void idra_nec_send_bit(int sBit)

    @Summary:
        Sends a 38KHz pulse train for a 1 or 0

    @Description:
        Generate a 1 or 0 NEC encoded IrDA pulse train using PWM output . See 
        http://www.mcselec.com/index.php?option=com_content&task=view&id=223
        and http://www.vishay.com/docs/80071/dataform.pdf.

    @Precondition:
        IrDA initialization that maps PPS pin RPB7 to OC5
 
    @Paramteters
        @Parameter 1: bit value

    @Returns:
        None

    @Example:   
        @code
        {
            idra_nec_send_bit(sBit);
        }

    @Remarks:
        None
  *****************************************************************************/
static void idra_nec_send_bit(int sBit)
{
unsigned int t1_1, t1_2;
    t1_1 = ReadCoreTimer();
    SetDCOC5PWM(IRDA_38K_ON);            // Turn on LED 38KHz burst
    do
    {
        t1_2 = ReadCoreTimer();
        
    }while((t1_2 - t1_1) < 22400);     // 560 us
    SetDCOC5PWM(IRDA_38K_IDLE);
    if(sBit)                            // Send "1"
    {
        do
        {
            t1_1 = ReadCoreTimer();         // Turn off LED 38KHz
        }while((t1_1 - t1_2) < 67600);     // 1.69ms
    }
    else
    {
        do
        {
            t1_1 = ReadCoreTimer();         // Turn off LED 38KHz
        }while((t1_1 - t1_2) < 22400);     // 560 us        
    }    
}

/*******************************************************************************
    @Function:
        void idra_nec_send_code(unsigned int code)

    @Summary:
        Sends the 32 bit IrDA NEC protocol key code.

    @Description:
        The NEC encoding sends an 8 bit address byte followed by it one's
        complement then the 8 bit command byte followed by its complement. See 
        http://www.mcselec.com/index.php?option=com_content&task=view&id=223
        and http://www.vishay.com/docs/80071/dataform.pdf.

    @Precondition:
        IrDA initialization that maps PPS pin RPB7 to OC5 PWm
 
    @Parameters
        @Parameter 1: NEC key code

    @Returns:
        None

    @Example:
    @code
        {
            idra_nec_send_code(key);
        }
  Remarks:
  *****************************************************************************/
void idra_nec_send_code(unsigned int code)
{
unsigned int t1_1, t1_2;    
int cntr = 31;
    PORTSetPinsDigitalOut(IOPORT_B, BIT_7); // Set IR_TX active
    idra_nec_send_sync();
    do
    {
        if(code & (1 << cntr))
        {
            idra_nec_send_bit(1);
        }
        else
        {
            idra_nec_send_bit(0);            
        }
        --cntr;
    }while(cntr >= 0);
    t1_1 = ReadCoreTimer();
    SetDCOC5PWM(IRDA_38K_ON);            // Turn on LED 38KHz burst
    do
    {
        t1_2 = ReadCoreTimer();
        
    }while((t1_2 - t1_1) < 22400);     // 560 us
    SetDCOC5PWM(IRDA_38K_IDLE);         // Turn off LED 38KHz 
    PORTSetPinsDigitalIn(IOPORT_B, BIT_7); // IR_TX -Tristate LED output
}

/*******************************************************************************
  Function:
    void idra_nec_send_sync(void)

  Summary:
    Sends the sync 38KHz pulse train for the IrDA NEC protocol

  Description:
    Generate an NEC encoded IrDA pulse train using PWM output . See 
    http://www.mcselec.com/index.php?option=com_content&task=view&id=223 and
    http://www.vishay.com/docs/80071/dataform.pdf.

  Precondition:
    IrDA initialization that maps PPS pin RPB6 to INT1
 
  Parameter 1: Interval period
    None
  Returns:
    None

  Example:
    idra_nec_send_sync();

  Remarks:
  *****************************************************************************/
static void idra_nec_send_sync(void)
{
unsigned int t1_1, t1_2;
    t1_1 = ReadCoreTimer();
    SetDCOC5PWM(IRDA_38K_ON);            // Turn on LED 38KHz burst
    do
    {
        t1_2 = ReadCoreTimer();
        
    }while((t1_2 - t1_1) < 360000);     // 9.0 ms
    SetDCOC5PWM(IRDA_38K_IDLE);         // Turn off LED 38KHz 
    do
    {
        t1_1 = ReadCoreTimer();         
        
    }while((t1_1 - t1_2) < 180000);     // 4.5 ms
}
/*******************************************************************************
  Function:
    void __ISR(_EXTERNAL_1_VECTOR, IPL2SOFT) ext_int1_isr(void)

  Summary:
    Services a External Interrupt 1

  Description:
    Computes the time interval between consecutive negative edges and calls
    one of two IrDA decoding algorithms.
 
  Precondition:
    IrDA initialization that maps PPS pin RPB6 to INT1
 
  Parameters:
    None - ISR cannot have variables passed to it.

  Returns:
    None - ISR cannot return a value.
 
  *****************************************************************************/
void __ISR(_EXTERNAL_1_VECTOR, IPL2SOFT) ext_int1_isr(void)
{
unsigned long t1_1;                 // Current time of interrupt
static unsigned long t2_1;          // Time of previous interrupt
unsigned long dt_1;                 // Time interval between edges
unsigned int bitCount = 0;          // Number of decoded bits
static int start_timing = 0;        // Time capture buffer have been initialized.

    t1_1 = ReadCoreTimer();
    if(!start_timing)               // Check for first interrupt after reset
    {
        t2_1 = t1_1;                // Initialize previous time on reset.
        start_timing = 1;
    }
    else
    {
        dt_1 = t1_1 - t2_1;         // Computer time interval
#ifdef IRDA_PULSE_LENGTH            // Call PULSE LENGTH decoder
        AIN1toggle();               // Toggle AIN1 output for timing
        bitCount = irda_sirc(dt_1);
#endif
#ifdef IRDA_NEC                     
        AIN1toggle();               // Toggle AIN1 output for timing
        bitCount = irda_nec(dt_1);  // Call NEC decoder
#endif
        t2_1 = t1_1;                // Update time of last interrupt
    }

    IFS0bits.INT1IF = 0;            // Clear the interrupt flag
}

void __ISR(_TIMER_2_VECTOR, IPL3SOFT) Timer2Handler(void)
{
//  mPORTBToggleBits(BIT_3);
// clear the interrupt flag
    mT2ClearIntFlag();
}

/* *****************************************************************************
 End of File
 */
