/* ************************************************************************** */
/** Descriptive File Name - Control of the IrDA module

  @Company
    Digilent

  @ Author
    Richard Wall
 
  @Date
    Created:    August 16, 2016
    Verified:   May 17, 2017

  @File Name
    irda.h

  @Summary
    IrDA driver functions header.

  @Description
    Decodes NEC or SIRC IrDA protocols. See Documentation.
 */
/* ************************************************************************** */

#ifndef _IRDA_H_    /* Guard against multiple inclusion */
    #define _IRDA_H_

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */
    
    #define IR_Bit              1
    #define IR_Out()            TRISGCLR = IR_Bit
    #define IR_RX               
    #define IR_pdown(a);        {if(a) LATGSET = IR_Bit; else LATGCLR = IR_Bit;}
    #define tris_IR_TX()        PORTSetPinsDigitalOut(IOPORT_B, BIT_7)
    #define tris_IR_RX()        PORTSetPinsDigitalIn(IOPORT_B, BIT_6)

    #define IRDA_HEADER_MARK    
    #define IRDDA_BIT_START     (int) (40000 * 0.55)   //0.55ms
    #define IRDA_ZERO_BLANK     (int) (40000 * 0.55)   //0.55ms
    #define IRDA_ONE_BLANK      (int) (40000 * 3 * IRDA_ZERO_BLANK)  //1.805ms
    #define IRDA_SLACK 100

    #define IRDA_38K_IDLE        0  // 264
    #define IRDA_38K_ON          23
    #define IRDA_38KHZ_GEN       262

    #define IRDA_NEC

    #define NEC_UP              (unsigned int) 0x20DFA25D //0x50A2DF20
#endif /* _IRDA_H_ */


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/*  IRDA Rx and Tx functions
*/

// *****************************************************************************
/**
  @Function
    void irda_init(void); 

  @Summary
    Initializes the IrDA Transceiver 

  @Description
    PORT B pin 7 is mapped to INT2 which is used to detect negative 
    transitions on the IrDA Rx line. 
    <p>
    PORT B pin 6 is used for the IrDA Tx line and is pulse width modulated.
    <p>
    This version is written for the NEC IrDA protocol
  @Precondition
    None
     
  @Parameters
    None
    
  @Returns
    None

  @Remarks
    None

  @Example
    @code
    {
      irda_init();
    }
*/
void irda_init(void); 

// *****************************************************************************
/**
  @Function
    int irda_nec(unsigned int dt_1); 

  @Summary
    Decodes a value of 1 or 0 for a NEC encoded signal.

  @Description
    Decodes a 32 bit value for a NEC encoded signal based on the period
    following the last 38KHz pulse. The global variable "new_key" is set = 1
    if 32 bits have been accumulated into the global variable "key". This 
    function must be called 32 times form the INT1 ISR to get a new key
    value.
    <p>
    PORT B pin 6 is used for the IrDA Rx line that generates an EXT INTR 1.
    <p>
    This version is written for the NEC IrDA protocol. See 
    http://www.vishay.com/docs/80071/dataform.pdf

  @Precondition
    The function irda_init must be executed to map the PPS pin for IR_TX.
     
  @Parameters
    @Param 1
        int dt_1 indicatine the number of core timer ticks since the last
        interrupt.
    
    @Returns
      The number of bits decoded.
  
    @Remarks
        Initializes the key code when a sync pulse is detected.
        <p>
        Any additional remarks.

    @Example
      @code
        {
            int x = irda_nec(dt_1);
        }
     */
    int irda_nec(unsigned int dt_1);
    
/*******************************************************************************
  @Function:  
    unsigned int irda_sirc(unsigned long dt_1)

  @Summary:
    Decodes IrDA Pulse-Length Coding. ALso known as SIRC infrared control code

  @Description:
    IrDA Pulse-Length Coding is described at following web sites.
        1. http://www.vishay.com/docs/80071/dataform.pdf
        2. http://www.st.com/content/ccc/resource/technical/document/application\
           note/c7/d1/63/f7/80/06/41/a4/CD00267896.pdf/files/CD00267896.pdf/jcr:\
           content/translations/en.CD00267896.pdf
 
  @Precondition:
    IrDA initialization that maps PPS pin RPB6 to INT1
 
  @Parameter 1: Interval period

  @Returns:
    Number of bits decoded.

  @Example:
    bit_cnt = unsigned int sirc( dt );

  @Remarks:
    The IrDA Pulse Length coding repeats the 20 bit code for the duration of
    the key held depressed 

    The last bit is not decoded because the bit value is in the length of the
    pulsing stream.  The detection uses the pulsing stream plus the inter-bit
    gap interval.  Since the next interrupt occurs at the start of the new
    key press, the last bit exceeds the ONE or ZERO bit timing. By observation,
    the last bit is always set to ONE. Hence when 19 bits are received, a ONE
    bit is added to the code.
  *****************************************************************************/
unsigned int irda_sirc(unsigned long dt_1);

// ************************************************************************
/**
  @Function
    void idra_nec_send_code(unsigned int code) 

  @Summary
    Sends an NEC encoded IrDA pulses

  @Description
    Sends the 38KHz pulse sequence for an NEC encoded key
    <p>
    PORT B pin 7 is used for the IrDA Tx line and is pulse width modulated
    using OC5.
    <p>
    This version is written for the NEC IrDA protocol
  @Precondition
    The function irda_init must be executed to map the PPS pin for IR_TX.
     
  @Parameters
    unsigned int code
    
  @Returns
    None
  
  @Remarks
    None

  @Example
    @code
      {
        idra_nec_send_code(code);
      }
*/
    void idra_nec_send_code(unsigned int code);

/*  End of File irda.h
 */
