/* ************************************************************************** */
/** Descriptive File Name:  main.c for Lab 7a. Generates one of eight single
                            frequency tones.

  @ Author
    Richard Wall
 
  @ Date
    Created;    April 30, 2016, 9:58 AM
    Modified:   February 19, 2017        
    Verified:   May 19, 2017

  @Company
    Digilent Inc

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @File Name
    main.c

  @Summary
    Audio tone generator

  @Description
    Generates one of eight single frequency sine waves based on the setting of
    SW0 through SW7. The tones range form 500 Hz to 8500 Hz. The tone is 
    produced for 5 seconds when BTND is pressed. The LCD and the 4-digit 7
    segment LED display show the value of the tone being generated. The tone
    is actively being generated when the 4-digit 7 segment display in lit. The
    audio volume is controlled by the speaker potentiometer.  

  @Remarks
    Uses XC32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
  
 ****   This application uses a peripheral bus clock set to 80MHz.  ****
    PBCLK set for 80 MHz, Set parameters in config_bits.h and hardware.h

 
 ******************************************************************************/

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */

// System included files
#include "config_bits.h"
#include "hardware.h"
#include <plib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

// Platform common included files
#include "swDelay.h"

// Application included files
#include "LCDlib.h"
#include "audio.h"
#include "led7.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

char LineOne[] = "Tone Generator";  // Sign on message
char frequency[16];         // String array containing the tone frequency
extern int16_t led_value;   // Value to be displayed on the 7 segment LED
extern BOOL led_flag;       // On / off control of the 7 segment LED

/** 
  @Function
    int main( void ); 

  @Summary
    Initializes the hardware platform and performs the calculator operations.

  @Description
 
    Configures the common Basys MX3 platform IO pins   

        Button Operations
     Button  -   Operation
     BTND   -   Generate tone
     BTNC   -   Not used
     BTNR   -   Not used
     BTNU   -   Not used - Up button (not functional in DEBUG mode)
     BTNL   -   Not used - Left button (not functional in DEBUG mode)

  @Precondition
    None

  @Parameters
    None

  @Returns
 The system has failed if the last statement is executed. 
      EXIT_FAILURE (1)   Indicates an error occurred
      EXIT_SUCCESS (0)   Indicates an error did not occur

  @Remarks
    1. No tone is generated until BTND is pressed. 
    2. A tone is generated when only one of the slide switches SW0 through SW7
       is set on.
    3. This implementation uses a PBCLK frequency of 80 Mhz as specified by the
       FPBDIV in config_bits.h
 */
int main(void)
{
int tone_sel;
int sw1 = 0;

    Hardware_Setup();
    initLCD();  
    audio_init();
    seg7_init();
    
    clrLCD();
    putsLCD( LineOne);
    setLCDC(LCD_LINE2);             // Position cursor to second line  
    putsLCD( "May 19, 2017");
    msDelay(1000);
    clrLCD();                       // Erase LCD
    putsLCD( "Set Switches");
    setLCDC(LCD_LINE2);             // Position cursor to second line
    putsLCD( "Push BTND");
    msDelay(100);
    putsLCD( "..now!");
   
    while(1)
    {
        if(BNTD())    
        {
            led_flag = TRUE;
            sw1 = Switch2Binary();
            switch(sw1)
            {
                case 1:
                    tone_sel = 1;
                    strcpy(frequency,"500 Hz");
                    led_value = 500;
                    break;
                case 2:
                    tone_sel = 2;
                    strcpy(frequency,"1500 Hz");
                    led_value = 1500;
                    break;
                case 4:
                    tone_sel = 3;
                    strcpy(frequency,"2500 Hz");
                    led_value = 2500;
                    break;
                case 8:
                    tone_sel = 4;
                    strcpy(frequency,"3500 Hz");
                    led_value = 3500;
                    break;
                case 16:
                    tone_sel = 5;
                    strcpy(frequency,"4500 Hz");
                    led_value = 4500;
                    break;
                case 32:
                    tone_sel = 6;
                    strcpy(frequency,"5500 Hz");
                    led_value = 5500;
                    break;
                case 64:
                    tone_sel = 7;
                    strcpy(frequency,"6500 Hz");
                    led_value = 6500;
                    break;
                case 128:
                    tone_sel = 8;
                    strcpy(frequency,"7500 Hz");
                    led_value = 7500;
                    break;
                default:
                    strcpy(frequency,"None Selected");
                    tone_sel = 0;
                    led_value = 0;
            }
            if(tone_sel)
            {
                start_tones(tone_sel);
                clrLCD();
                putsLCD( LineOne);
                gotoLCD(16);
                putsLCD(frequency);
                msDelay(5000);
            }
            led_flag = FALSE;
            start_tones(0);
            msDelay(50);
            while(BNTD());  // Debounce BTND
        }       
    }
    return (EXIT_FAILURE);  // System failure if the code execution gets here.
}

// End of main.c
