/* ************************************************************************** */
/** Descriptive File Name:   main.c for Lab 7B that implements a spectrum 
        analyzer

  @ Author
    Richard Wall
 
  @ Date
    Created;    April 16, 2014
    Modified:   December 30, 2016, PIC32MX370 target processor
                January 4, 2017 - Uses MIPS FFT
    Verified:   May 20, 2017

  @Company
    Digilent Inc

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @File Name
    main.c

  @Summary
    Spectrum Analyzer

  @Description
   Samples of the amplified audio input from the platform microphone are batch
   processed with a Fast Fourier Transform algorithm using the mips DSP library. 
   The character LCD on the Basys MX3 platform is used to display the frequency
   bins (buckets). The frequency of the greatest amplitude signal is displayed
   on the four digit seven segment LED display. Since the sensitivity of the
   microphone circuit is controlled by the MIC VOLUME pot, the amplitude of the
   frequency bins are relative and are linear.

  @Remarks
    Uses XC32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
 
  ****   This application uses a peripheral bus clock set to 80MHz.  ****
    PBCLK set for 80 MHz, Set parameters in config_bits.h and hardware.h

 **************************************************************************** */

// System included files 
#include "config_bits.h"    // Processor configuration
#include "hardware.h"       // Platform declarations

// XC32 libraries
#include <plib.h>
#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include <dsplib_dsp.h>

// Included files for platform 
//#include "main.h"
#include "LCDlib.h"
#include "led7.h"
#include "comm_lib.h"
#include "swDelay.h"

// APplication included files
#include "micInput.h"
#include "fft.h"

// Variables declared and used in led7.c needed for Timer 1 used for 7 segment 
// display control
extern volatile int16_t led_value;  // 4 digit value to be displayed on 7 Seg.
extern volatile BOOL led_flag;      // 7 Segment LED  active control flag

// Variables declared  and used in micInput.c needed for data sampling 
extern volatile int16_t buffer1[];  // sampled data buffer #1
extern volatile int16_t buffer2[];  // sampled data buffer #2
extern volatile int adc;            // Latest ADC10 sample
extern volatile int active_buffer;  // Which buffer is actively being loaded
extern volatile int buffer_count;   // Active buffer index
extern volatile BOOL buffer_switch; // Just switched active buffer flag
extern volatile BOOL buffer_processing_complete; // Buffer has been off loaded

// Variables declared  and used in fft.c needed for FFT computation 
extern int16c InData[];         // Time domain input signal
extern int16c OutF[];           // Frequency domain FFT result
extern int16c OutT[];           // and inverse FFT back to the time domain
extern int16  BlackWindow[];    // Our windowing function

// FFT conversion for display
int16_t amp[FFTLEN/2];  // Frequency buckets - Absolute value
int16_t lg[FFTLEN/2];   // Scaled amplitude for LCD display

// Local function prototypes 
static void initialize(void);   // Hardware platform configuration
static int peak_detector(int16_t *pwr, int N); // Detects max amplitude F bin

/* **************************************************************************
 * This is the entry to main process execution loop. After initialization, all 
 * displays and outputs are updated approximately every 50 ms. The FFT algorithm 
 * does not use a conversion in place so the input data set is retained during
 * and after conversion. Since the real data set represents a one dimensional
 * FFT, the imaginary data needs to be set to zero only once during 
 * initialization.  
 * 
 * If data windowing is not used and the FFT phase is not required,
 * no floating point libraries are needed. 
 **************************************************************************** */
int main (void)
{
int i;

    initialize();               /* Initialize hardware */
#ifndef NO_FLOAT
    WindowBlackmanHarrisInit(BlackWindow, (int) FFTLEN);
#endif
    
// Clear the imaginary input data array.
    for(i=0; i<FFTLEN ; i++)
    {
        InData[i].im = 0;
    }

// Initialize LCD display 
    clrLCD();
    putsLCD( "LCD Spectrum");
    setLCDC(LCD_LINE2);         // Position cursor to second line 
    putsLCD( "May 19, 2017");
    msDelay(2000);
    clrLCD();
    putsLCD( "Richard Wall");
    setLCDC(LCD_LINE2);         // Position cursor to second line 
    putsLCD( "Basys MX3");
    msDelay(2000);
    clrLCD();
    while(1)                    // Infinite operational process loop 
    {
        if(buffer_switch == TRUE)
        {
            AIN1toggle();       // Display update timing output
            if(active_buffer == 1 )
            {
                for(i=0; i<FFTLEN; i++)
                {
                    InData[i].re = buffer2[i];
                }
            }
            else
            {
                for(i=0; i<FFTLEN; i++)
                {
                    InData[i].re = buffer1[i];
                }
            }
            /*
// Spectrum data sent to serial port
            printf("%T: %4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d\n\r",
                   InData[0].re,InData[1].re,InData[2].re,InData[3].re,
                   InData[4].re,InData[5].re,InData[6].re,InData[7].re,
                   InData[8].re,InData[9].re,InData[10].re,InData[11].re,
                   InData[12].re,InData[13].re,InData[14].re,InData[15].re); 
            */
            
#ifndef NO_FLOAT
            WindowBlackmanHarris(InData, FFTLEN); // Data window
#endif
            BIN1(1);                    // FFT execution timing output
            FFT(OutF, InData);          // Mips DSP based FFT
            BIN1(0);                    // FFT execution timing output
            
            for(i=0; i<FFTLEN/2; i++)   // Convert from complex to polar and
            {                           // Scale for LCD display
                amp[i] = FFTAmplitude(OutF[i].re, OutF[i].im);
                lg[i] = FFTLog(amp[i]); // Not really a log function
            }
            FFT_Disp(lg, FFTLEN/2);     // Update LCD
            peak_detector(lg, FFTLEN/2); // Pick max amplitude frequency bucket
            msDelay(46);                // Display loop preemptive delay
            buffer_switch = FALSE;      // Reset switched buffer flag
            buffer_processing_complete = TRUE; // Buffer is free for loading
        }
    }
    return EXIT_FAILURE;   /* We'll never get here - hopefully */
} /* End of main */


/* initialize  FUNCTION DESCRIPTION *************************************
SYNTAX:         static void initialize(void);
KEYWORDS:       initialization system
DESCRIPTION:	Sets up for all the Basys MX3 IO
RETURN VALUE:   none
NOTES:          Updated to Basys MX3
END DESCRIPTION **********************************************************/
static void initialize(void)
{
    Hardware_Setup();    // Initialize processor board 
    initiate_ADC();

// Initialize HMI 
// Character LCD
    uart4_init(38400, NO_PARITY); // Can be used with printf functions
    initLCD();
    initLCDGr();

 // 4 digit 7 segment display - uses Timer 1 to scan the digits
    seg7_init();
    test_7seg_leds();
    msDelay(500);
    clr_dsp();
    DIGITS_OFF();

    led_value = 0;  // Turn off 7 segment LED display
    led_flag = 1;
    
}  // End of initialize 

/* peak_detector  FUNCTION DESCRIPTION *************************************
SYNTAX:         static int peak_detector(int16_t *pwr, int N);
KEYWORDS:       peak, detector
DESCRIPTION:	Selects largest value from an array of positive integers. 
 * Parameter 1: integer array of values
 * Parameter 2: Size of array
RETURN VALUE:   The index of the element of the greatest magnitude
NOTES:          Turns on 7 segment LED display and displays the frequency
                associated with that index.
END DESCRIPTION **********************************************************/
static int peak_detector(int16_t *pwr, int N)
{
int i, pki = 0;
int pk = 0;
    for(i=1; i<N; i++)      // Scan for greatest amplitude
    {
        if(pwr[i]>pk)
        {
            pk = pwr[i];
            pki = i;
        }
    }
    led_value = pki*FS/(N*2);   // Set 7 Segment LED display value - Hz
    led_flag = 1;               // Enable 7 Segment LED display
    return pki;
}
// End of main.c 
