/* ************************************************************************** */
/** Descriptive File Name: micInput.c for Lab 7b

  @Company
    Digilent Inc

  @Author
    Richard Wall

  @Date
    Created:    December 5, 2016
    Revised:    January 5, 2017
    Verified:   May 19, 2017

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @File Name
    micInput.c

  @Summary
   Read the analog input attached to the microphone amplifier.  

  @Description
 * This provides for a single channel conversion that is triggered by a Timer 3.
 * Two buffers are loaded with data in a ping-pong manner to allow processing
 * of one buffer while the other is being loaded. This function is meant to be
 * used with a block processing algorithm.
 */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

// System included files
#include "hardware.h"
#include <plib.h>
#include <math.h>
#include <stdint.h>

// Platform common included files
#include "swDelay.h"

// Application included files
#include "main.h"
#include "fft.h"
#include "micInput.h"


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */
/** Descriptive Data Item Name

  @Summary
 * Global variable "adc" is the most recent ADC conversion. Two data buffers
 * are loaded in a ping-pong fashion. Flags are set to indicate which buffer
 * has been loaded and when it has happened. 
    
  @Description
 * These flags are used by and changed by the main processing loop.
 */
int adc;
int active_buffer = 1;                      // Which buffer
BOOL buffer_switch = FALSE;                 // Buffers switched
BOOL buffer_processing_complete = TRUE;     // Allows sampling
int16_t buffer1[FFTLEN], buffer2[FFTLEN];   // ping-pong buffers

/* initiate_ADC FUNCTION DESCRIPTION *************************************
SYNTAX:         int initiate_ADC(void);
KEYWORDS:       PIC32, ADC10, initialization
DESCRIPTION:	Configures AN4 for analog input.
RETURN VALUE:   none
NOTES:          The analog conversion is started by a Timer 3 roll over. After
 *              conversion, the ADC automatically reverts to sample mode.
 *
 *              Minimum Sample tome: TSAM = 200ns * PBCLK. For PBCLK = 80 MHz,
 *              set ADC_SAMPLE_TIME_16 or greater 
 * 
 *              The ADC clock period is determined by (int) (65 ns * 2)/PBCLK
 *              rounded up. For PBCLK = 80 MHz, ADC_CONV_CLK = 11
 * 
 *Reference:    http://ww1.microchip.com/downloads/en/DeviceDoc/61104E.pdf
 *               
END DESCRIPTION **********************************************************/
void initiate_ADC(void)
{
// Define setup parameters for OpenADC10
//                Turn module on | output in integer | trigger mode auto | enable auto sample
// #define PARAM1    ADC_MODULE_ON | ADC_FORMAT_INTG   | ADC_CLK_AUTO      | ADC_AUTO_SAMPLING_ON // Auto restart   
 #define PARAM1    ADC_MODULE_ON | ADC_FORMAT_INTG32 | ADC_CLK_TMR       | ADC_AUTO_SAMPLING_ON // Timer 3 start

             // ADC ref Vdd/Vss    | disable offset test    | disable scan mode | perform 1 sample | use dual buffers | use alternate mode
#define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF
 
//          use ADC internal clock | set sample time 
#define PARAM3  ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_20 | ADC_CONV_CLK_12Tcy
// #define PARAM3  ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_5 | ADC_CONV_CLK_Tcy2  // +- 200

// set AN4 as analog input for mic 
#define PARAM4  ENABLE_AN4_ANA

    // do not assign channels to scan    
#define PARAM5	SKIP_SCAN_ALL

// Set Port B Pins 2 or 4 as analog Input 
    ANSELBbits.ANSB2 = 1;       // POT input
    ANSELBbits.ANSB4 = 1;       // mic input

// Disable ADC before configuration 
    CloseADC10();   /* Ensure the ADC is off before setting the configuration */
    
// configure to sample AN4
    SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN4); // mic input
//  SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN2); // POT input

    OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 );

// Enable ADC complete interrupt 
    ConfigIntADC10(ADC_INT_ON | ADC_INT_PRI_4 | ADC_INT_SUB_PRI_1);
    
// Enable ADC */
    EnableADC10();      // Enable the ADC 

// Timer 3 is the only 16 bit timer that can be used to auto - initiate an ADC
// conversion 
    OpenTimer3(T3_ON | T3_SOURCE_INT | T3_PS_1_1, (SAMPLE_INTERVAL - 1));

// Configure timer interrupt with a priority of 2
    ConfigIntTimer3(T3_INT_ON | T3_INT_PRIOR_3 | T3_INT_SUB_PRIOR_1);
    
// Wait for the first conversion to complete so there will be valid
// data in ADC result registers. This statement is required only if the
// ADC operation is manually operated. */
//	Read_ADC(0);
	
} // End initiate_ADC/

/* ************************************************************************** */
/* Read_ADC FUNCTION DESCRIPTION **************************************
SYNTAX:         unsigned int Read_ADC(int channel);
KEYWORDS:       PIC32 ADC10 read channel
DESCRIPTION:	Reads a single channel into the buffer specified. This function
                that reads the value of the buffer for the given argument.
                Value must be 0-15 or 0x0 to 0xF for the 16 buffers.
PARAMETER 1:	AD Channel
RETURN VALUE:   unsigned int - digital value for the conversion result in the 
                range of 0 to 1023.
NOTES:          This function is require if the ADC operation is manually
                initiated in the function that initializes the ADC . 
 * Refer to http://ww1.microchip.com/downloads/en/DeviceDoc/61104E.pdf
END DESCRIPTION **********************************************************/
unsigned int Read_ADC(int channel)
{
int16_t adc;

    AD1CON1bits.DONE = 0;
    AcquireADC10();
    while(!BusyADC10())
    {
/* Wait for the conversion to complete so there will be vaild data in
 * ADC result registers */
    }
    adc = ReadADC10(channel);
    if(adc>1023)            // Limit the ADC output to 2^10 - 1.
        adc = 1023;
    return adc;
} /* End Read_ADC */

/* T3Interrupt Handler FUNCTION DESCRIPTION **********************************
 * SYNTAX:      T3Interrupt(void);
 *
 * KEYWORDS:    hardware delay, interrupt, Timer 3, Sample rate
 *
 * DESCRIPTION: This ISR initiates an ADC conversion. No action is completed
 *              here.
 *  
 * PARAMETERS:  None
 * 
 * RETURN VALUE: None
 *
 * Notes:       This works only if the ISRfunction requires less than interrupt
 *              period.
 * END DESCRIPTION **********************************************************/
void __ISR(_TIMER_3_VECTOR, IPL3SOFT) T3Interrupt(void)
{
    INTClearFlag( INT_T3 );         /* Clear the T3 interrupt flag */
} /* T3Interrupt Handler */

/* ADCInterrupt FUNCTION DESCRIPTION *****************************************
 * SYNTAX:      void __ISR(_ADC_VECTOR, IPL3SOFT) ADCInterrupt(void);
 *
 * KEYWORDS:    Analog to Digital Conversion, ISR, Interrupt
 *
 * DESCRIPTION: This ISR is activated by an ADC completed conversion. Buffers 
 *              are filled in a ping-pong fashion. Global variable flags are
 *              set to indicate the status of the buffer activity.
 *
 * RETURN VALUE: None
 *
 * Notes:       This works only if the ISR function requires less than
 *              the interrupt period set by Timer 3.
 * 
 * END DESCRIPTION **********************************************************/
void __ISR(_ADC_VECTOR, IPL4SOFT) ADCInterrupt(void)
{
static int cnt = OVER_SAMPLE_RATE;
static int buffer_count = 0;
unsigned int adc0; 

    AIN2toggle();   		// Sample rate timing instrumentation output
    BIN2(1);				// Set processing instrumentation timing output
    adc0 = ReadADC10(0);	// For ADC interrupt only - read ADC buffer 0 only

    if(--cnt == 0)          // Check buffer index
    {
        adc = (int16_t)(((int) adc0) - 511); // Convert to signed SIGNED SHORT
        cnt = OVER_SAMPLE_RATE;
        switch (active_buffer)      // determine which buffer to load 
        {
            case 1:
                if (buffer_count < FFTLEN) // test for data block filled 
                {  // Save sample to buffer 1 and increment index
                    buffer1[buffer_count++] = adc;  
                }
                else
                {  // Switch buffer only if alternate buffer has been processed
                    if(buffer_processing_complete == TRUE)
                    {
                        buffer_count = 0;       // Reset buffer index
                        active_buffer = 2;      // Set buffer2 to active
                        buffer_switch = TRUE;   // Set buffer loaded flag
// Reset processing complete flag to indicate that ready for next processing
                        buffer_processing_complete = FALSE; 
                    }
                }
                break;
            case 2:
                if (buffer_count < FFTLEN)
                {   // Save sample to buffer 2 and increment index
                    buffer2[buffer_count++] = adc; 
                }
                else
                {  // Switch buffer only if alternate buffer has been processed 
                    if(buffer_processing_complete == TRUE)
                    {
                        buffer_count = 0;       // Reset buffer index
                        active_buffer = 1;      // Set buffer2 to active
                        buffer_switch = TRUE;   // Set buffer loaded flag
// Reset processing complete flag to indicate that ready for next processing
                        buffer_processing_complete = FALSE;
                    }
                }
                break;
            default:                            // Reset all buffer controls
                buffer_count = 0;
                active_buffer = 1;
                buffer_switch = FALSE;
                buffer_processing_complete = TRUE;
                break;
        }
    }
    BIN2(0);	// Clear processing instrumentation timing output
    INTClearFlag( INT_AD1 );    // Must clear interrupt flag
} // End of ADCInterrupt handler 


/* ****************************************************************************
 End of micInput.c file
 */
