/* ************************************************************************** */
/** Descriptive File Name: RC.c - Remote servo controller demonstration program

  @ Author  
     Richard Wall
  
  @ Date: 
     Created:    November 24, 2012
     Revised:    March 3, 2017 - ported to PIC32MX370 for the Basys MX3 
     Verified:	 May 21, 2017
	 
  @Company
    Digilent Inc

  @File Name
    RC.c

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    Test program for the 2 channel Servo Motor using the Basys MX3.

  @Description
    See RC.txt under the Reference folder

  @ Remarks:
    An external 5VDC is required for servo power
    
    This implementation uses PWM outputs to generate the servo proportional
    position signal.
**************************************************************************** */

// System included files
#include "config_bits.h"    // Processor configuration
#include "hardware.h"       // Platform declarations

// XC32 libraries
#include <plib.h>
#include <string.h>
#include <stdio.h>
#include <stdint.h>

// Included files for platform 
#include "LCDlib.h"
#include "comm_lib.h"
#include "swDelay.h"

// Included files for application 
#include "RC.h"

int rc[NRC];		// Array of RC Channel position Range = 0-100
char BLANK_LINE[] = "                    ";

/* ----------------------------------------------------------- */
int main(void)
{
char lcdstr[40];                // Local LCD display buffers
char uartstr[80];               // Local UART input buffer
unsigned int ch, pos, rc_pwm;	// Used for UART entry

	Hardware_Setup();           // Initialize common Basys MX3 hardware
	initLCD();                  // Initialize LCD
	uart4_init(38400, NO_PARITY);	// Initialize UART
	initRC();                   // Initialize RC channel output pins
	clrLCD();                   // Clear hello messages
	putsLCD("RC Demo");         // Hello messages
	printf("\n\rDr. Wall's RC Demo\n\r");
	printf("Enter servo channel and position\n\r");
	msDelay(2000);              // Wait a second or two
	check_switches();           // Initialize RC times
	clrLCD();                   // Clear hello messages
	while(1)
	{
        if(getstrU4( uartstr, strlen(uartstr) ) > 0)
        {
            gotoLCD(16);		// Blank out LCD line 1
            putsLCD(BLANK_LINE);
        	gotoLCD(16);		// Blank out LCD line 1
    		putsLCD(uartstr);	// Copy string to LCD line 2
            sscanf(uartstr,"%d %d",&ch, &pos); // Decode entry
            if(ch >= 1 && ch <= NRC)	// Check RC channel number range
        	{
    			if(pos >=0 && pos <= 100)	// Check RC position range
                {
                    rc_pwm = RC_MIN + (pos*RC_SPAN/100); 
            		switch(ch)
                    {
                        case 1:
                            rc[0] = pos;
                            RC_1(rc_pwm); // Update RC channel position
                            break;
                        case 2:
                            rc[1] = pos;
                            RC_2(rc_pwm); // Update RC channel position
                            break;
                    }
                    printf("\n\rChannel: %d  Pos:  %d  PWM: %d\n\r", ch, pos, rc_pwm);
                }
            }		
    	}
	}
	return (EXIT_FAILURE);		// The program should never get to this LOC 
}


/* START FUNCTION DESCRIPTION ********************************************
system_init
SYNTAX:  		void initTimer2{void);
KEYWORDS:		initialization system timer 2
DESCRIPTION:	Sets timer 2 for an 10 us interrupt
RETURN VALUE:   none
NOTES:			See RC.h for pin definitions
END DESCRIPTION **********************************************************/
void initRC(void)
{
#define TMR2_TICK	25000

	cfgRC1();		// Set RC pins for PWM output
	cfgRC2();		

    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_4, PWM_PD-1);
	// Set Timer 2 interrupt with a priority of 2
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
    
    OpenOC5((OC_ON|OC_TIMER_MODE16|OC_TIMER2_SRC|OC_PWM_FAULT_PIN_DISABLE), 
            RC_CENTER, RC_CENTER);  // RC1
    OpenOC4((OC_ON|OC_TIMER_MODE16|OC_TIMER2_SRC|OC_PWM_FAULT_PIN_DISABLE), 
            RC_CENTER, RC_CENTER);  // RC2   
// configure to sample AN2 as the speed control
   SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN2); 
// configure ADC using parameter define above
   OpenADC10( ADC_PARAM1, ADC_PARAM2, ADC_PARAM3, ADC_PARAM4, ADC_PARAM5 ); 
   EnableADC10();      // Enable the ADC
    
}

/* START FUNCTION DESCRIPTION ********************************************
Timer2Handler
SYNTAX:			Timer2Handler(void);
KEYWORDS:		hardware delay, interrupt, Timer 2
DESCRIPTION:   	Processes the code associated with a Timer 2 interrupt.
				It is responsible for generating the signals for each 
				RC channel in two phases: The three state FSM control the
				on period and the off period.
PARAMETERS:		None are allowed for an ISR
RETURN VALUE:  	None are allowed for an ISR
Notes:			This function cannot be called from any other function.
END DESCRIPTION **********************************************************/
void __ISR(_TIMER_2_VECTOR, IPL2SOFT) Timer2Handler(void)
{
	check_switches();		// Initialize RC times
    mT2ClearIntFlag();			// clear the interrupt flag
}

/* START FUNCTION DESCRIPTION ********************************************
SYNTAX:			int check_buttons(void);
KEYWORDS:		buttons, decode
DESCRIPTION:   	This function polls switches SW0 through SW2 on the 
				Basys MX3 board and sets the RC positions
				to a common value. The RC channel positions are not changed
				unless there has been a change of switch state.   
PARAMETERS:		None
RETURN VALUE:  	1 = new button condition, 0 = no new button condition
Notes:			The button(s) pressed are encoded using a 1 hot code.  
				SW0 = 0/1, SW1 = 0/2, and SW2 = 0/4;
END DESCRIPTION **********************************************************/
void check_switches(void)
{
char lcdstr[40];
static int s2 = 0xff;	// force setting RC channels when initialized
BOOL update = FALSE;
int s1; 		// b1 does not need to be static - the newest status
int setting;	// RC servo position setting
int offset;
int adc2;
int pos;
	
    setLED0(SW0());
    setLED1(SW1());
    setLED2(SW2());
    setLED2(SW7());
	s1 = Switch2Binary();	

    if(SW7())
    {
        offset = 8 * ((~ReadActiveBufferADC10() & 0x01));  
        adc2 = ReadADC10(offset);    
        pos = (adc2 * 100) / 1024;
        setting = (pos * RC_SPAN / 100) + RC_MIN;
        rc[0] = pos;
        rc[1] = pos;
        msDelay(5);
        update =  TRUE;
    }
    else
    {
        if(s1 != s2)            // Is this a new button setting?
        {
            switch(s1 & 0x07)	// Set all RC units to the same setting
    		{
        		case 0: 
                    pos = 0;
                	break;
    			case 1:				
                    pos = 25;
            		break;
                case 2:				
                    pos = 50;
        			break;
            	case 3:				
                    pos = 75;
                    break;
    			case 4:				
                    pos = 100;
            		break;
                default:
                    pos = 0;
    		}
            update =  TRUE;
        }
	}
    if(update)
    {
    	setting = RC_MIN + ((RC_SPAN * pos) / 100);
        rc[0] = pos;
        rc[1] = pos;
        RC_1(setting);
        RC_2(setting);

        sprintf(lcdstr,"CH1:%3d CH2:%3d",rc[0], rc[1]);
      	gotoLCD(0);			// Blank out LCD line 1
        putsLCD(BLANK_LINE);
        gotoLCD(0);
        putsLCD(lcdstr);	// Post channel settings to LCD line 1
        if(!SW7())
        {
            printf("\n\r%s\n\r", lcdstr);	// Send channel setting to UART
        }
        update =  FALSE;
    }
    s2 = s1; 
}
